@@ -49,7 +49,7 @@ char *__xpg_strerror_r(int, char *, size_t);
 #endif
 
 static const struct got_error got_errors[] = {
-	{ GOT_ERR_OK,		"no error occured?!?" },
+	{ GOT_ERR_OK,		"no error occurred?!?" },
 	{ GOT_ERR_ERRNO,	"see errno" },
 	{ GOT_ERR_NOT_GIT_REPO, "no git repository found" },
 	{ GOT_ERR_BAD_FILETYPE,	"bad file type" },
@@ -91,7 +91,6 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_PACK_OFFSET,	"bad offset in pack file" },
 	{ GOT_ERR_OBJ_EXISTS,	"object already exists" },
 	{ GOT_ERR_BAD_OBJ_ID,	"bad object id" },
-	{ GOT_ERR_ITER_BUSY,	"iteration already in progress" },
 	{ GOT_ERR_ITER_COMPLETED,"iteration completed" },
 	{ GOT_ERR_RANGE,	"value out of range" },
 	{ GOT_ERR_EXPECTED,	"expected an error but have no error" },
@@ -107,7 +106,7 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_BAD_REF_DATA,	"could not parse reference data" },
 	{ GOT_ERR_TREE_DUP_ENTRY,"duplicate entry in tree object" },
 	{ GOT_ERR_DIR_DUP_ENTRY,"duplicate entry in directory" },
-	{ GOT_ERR_NOT_WORKTREE, "no got work tree found" },
+	{ GOT_ERR_NOT_WORKTREE, "no work tree found" },
 	{ GOT_ERR_UUID_VERSION, "bad uuid version" },
 	{ GOT_ERR_UUID_INVALID, "uuid invalid" },
 	{ GOT_ERR_UUID,		"uuid error" },
@@ -243,6 +242,8 @@ static const struct got_error got_errors[] = {
 	{ GOT_ERR_MERGE_COMMIT_OUT_OF_DATE, "merging cannot proceed because "
 	    "the work tree is no longer up-to-date; merge must be aborted "
 	    "and retried" },
+	{ GOT_ERR_BUNDLE_FORMAT, "unknown git bundle version" },
+	{ GOT_ERR_BAD_KEYWORD, "invalid commit keyword" }
 };
 
 static struct got_custom_error {
