@@ -52,7 +52,7 @@ const struct got_error *got_pack_parse_object_type_and_size(uint8_t *,
 
 #define GOT_PACK_PREFIX		"pack-"
 #define GOT_PACKFILE_SUFFIX	".pack"
-#define GOT_PACKIDX_SUFFIX		".idx"
+#define GOT_PACKIDX_SUFFIX	".idx"
 #define GOT_PACKFILE_NAMELEN	(strlen(GOT_PACK_PREFIX) + \
 				SHA1_DIGEST_STRING_LENGTH - 1 + \
 				strlen(GOT_PACKFILE_SUFFIX))
@@ -222,5 +222,5 @@ const struct got_error *got_packfile_extract_object(struct got_pack *,
 const struct got_error *got_packfile_extract_object_to_mem(uint8_t **, size_t *,
     struct got_object *, struct got_pack *);
 const struct got_error *got_packfile_extract_raw_delta(uint8_t **, size_t *,
-    size_t *, off_t *, off_t *, struct got_object_id *, uint64_t *, uint64_t *,
-    struct got_pack *, struct got_packidx *, int);
+    size_t *, off_t *, off_t *, off_t *, struct got_object_id *, uint64_t *,
+    uint64_t *, struct got_pack *, struct got_packidx *, int);
